// SubDirectory.cpp : implementation of the SubDirectory class

#include <stdlib.h>
#include <stdio.h>
#include "Constants.h"
#include "BitFlagg.h"
#include "CountFilesSumBytes.h"
#include "ReturnValues.h"
#include "ReturnSum.h"
#include "SubDir.h"
#include "SubDirectory.h"
#include "FilesInDir.h"

SubDirectory::SubDirectory (string Dirname, Cnfg & Configg)
{
	msPathInput = Dirname;
	msSubDirectname = Dirname;
	process (Configg);
}

void SubDirectory::process (Cnfg & Configg)
{
	Mntlist = new MountArray (Configg.RecurseMounts);
	string DebugAssist, Path_;		// Path_ in SaveDirectory INfo
	int CounterSaveDir = 0;				// Counter for save Directory info

	msErrorString = "";						//

	FilesInDir *fid = new FilesInDir (msPathInput, Mntlist);

	if (fid->open) {
//  Count the Directories    CountDirectories
		miCountDirs = fid->CountDirs ();
		mpRetVal = new ReturnValues *[miCountDirs];
		fid->rewind ();
//  miCountDirs is now the number of subdirectories in the starting directory
		mpReturnSum = new ReturnSum;
		mpCFSB = new CountFilesSumBytes;
		*mpReturnSum += mpCFSB->Cfsb (msPathInput);
		if (miCountDirs == 0) {
			miNumberofLines = miCountDirs;
			msHeader[1] += "No subdirectories to analyze";
		} else											// if count dirs > 0
		{														//  Save the required information
			string EntryName;

			while (fid->NextDir (EntryName)) {
				mpRetVal[CounterSaveDir] = new ReturnValues;
				string fullpath;

				if (msPathInput == "/")
					fullpath = "/" + EntryName;
				else
					fullpath = msPathInput + "/" + EntryName;

				// Search all of the Subdirectories of this Subdirectory
				SubDir *pSubSubD = new SubDir (fullpath, msErrorString, Mntlist);

				// Count Files and Bytes of this SubDirectory and add with all SubSubs
				// Save in Return Array and add to return Sum

				(*mpReturnSum) += (*mpRetVal[CounterSaveDir]) =
					pSubSubD->return_values + mpCFSB->Cfsb (fullpath);
				mpRetVal[CounterSaveDir]->msDirname = EntryName;

				CounterSaveDir++;
				delete pSubSubD;
			}													// while
		}														// else countdirs
//  End of Save the required information  
		delete mpCFSB;

		SortDirectoryInfo ();
		OutputDirList ();						// output a list of the saved dir's
	}															// if else opendir
	delete fid;
}


SubDirectory::~SubDirectory ()
{
	int counter;

	for (counter = 0; counter < miCountDirs; counter++) {
		delete mpRetVal[counter];
		delete mpOutputString[counter];
	}
	delete mpRetVal;
	delete mpOutputString;
	delete mpReturnSum;
	delete mBitFlag;
	delete Mntlist;
}

void SubDirectory::SortDirectoryInfo ()
{
	int a, b;
	ReturnValues buffer;

	for (a = 1; a < miCountDirs; a++) {
		for (b = 0; b < miCountDirs - 1; b++) {
			if (++(*mpRetVal[b]) < ++(*mpRetVal[b + 1])) {
				buffer = *mpRetVal[b];
				*mpRetVal[b] = *mpRetVal[b + 1];
				*mpRetVal[b + 1] = buffer;
			}
		}
	}
}

//function to output Create String array

void SubDirectory::OutputDirList ()
{
	int counter, files, totfiles;
	double percent;
	string stringg[4], StringTemp, tab, ttab;

	tab = "\t";
	ttab = "\t\t\t";
	ulnglng sum;
	ulnglng denom;
	char StringConv[120], sc1[80], sc2[80], sc3[80];

	mpOutputString = new string *[miCountDirs + 1];
	for (counter = 0; counter < miCountDirs + 1; counter++) {
		mpOutputString[counter] = new string;
		*mpOutputString[counter] = "";
	}
	mBitFlag = new BitFlagg (miCountDirs);
	miNumberofLines = miCountDirs + 1;

	denom = mpReturnSum->mlNetTotal;

	if (!denom) {									// if( denom )
		msErrorString = "Zero bytes; avoiding division by zero.";
		percent = 0;
	} else
		percent = (100 * (double) mpReturnSum->mlBytes / (double) denom);	//(double)

	sprintf (StringConv, "%s has %lli Bytes in %i Files, with %i Subdirector",
					 msPathInput.c_str (), mpReturnSum->mlBytes, mpReturnSum->miFiles,
					 miCountDirs);
	msHeader[0] = StringConv;
	if (miCountDirs == 1)
		msHeader[0] += "y";
	else
		msHeader[0] += "ies";
	sprintf (StringConv, "\nwhich is %5.2f percent of total", percent);
	stringg[3] = StringConv;
	msHeader[0] += stringg[3];

	totfiles = mpReturnSum->miFiles;
	sprintf (StringConv,
					 "   Subdirs   This Directory         1 Deeper         2 - %i Deeper          Totals",
					 LEVELS);
	msHeader[2] = StringConv;
	msHeader[3] =
		"      Name   Bytes   Files   Dirs   Bytes   Files    Dirs   Bytes   Files   Files   Bytes    %";

	for (counter = 0; counter < miCountDirs; counter++) {
		ReturnValues *lrvp = mpRetVal[counter];

		sum = ++(*lrvp);
		files = ~(*lrvp);
		totfiles += files;
		if (sum > denom) {
			int dummy;

			dummy++;
		}
		if (!denom) {
			msErrorString = "Zero bytes; avoiding division by zero.";
			percent = 0;
		} else
			percent = (100 * (double) sum / (double) denom);	//(double)
		sprintf (StringConv, "%10.10s %7.7s %7i", lrvp->msDirname.c_str (),
						 conv (lrvp->mlTotal, sc1), lrvp->miCount);
		StringTemp = StringConv;

		if (lrvp->miSubDir) {
			sprintf (StringConv, "%7i %7.7s %7i ", lrvp->miSubDir,
							 conv (lrvp->mlSTotal, sc2), lrvp->miSFiles);
		} else {
			sprintf (StringConv, "%24.24s", "");
		}
		StringTemp += StringConv;
		if (lrvp->miSSubDir) {
			sprintf (StringConv, "%7i %7.7s %7i ", lrvp->miSSubDir,
							 conv (lrvp->mlSSTotal, sc2), lrvp->miSSFiles);
		} else
			sprintf (StringConv, "%24.24s", "");
		StringTemp += StringConv;

		sprintf (StringConv, "%7i %7.7s %5.2f", files, conv (sum, sc2), percent);
		StringTemp += StringConv;

		if ((lrvp->miSSubDir) || (lrvp->miSubDir))
			mBitFlag->Store (counter, TRUE);
		else
			mBitFlag->Store (counter, FALSE);
		*mpOutputString[counter] = StringTemp;
	}
	msHeader[1] = msErrorString;
	StringTemp = "";
	sprintf (StringConv, "%10.10s %7.7s %7i", "Totals",
					 conv (mpReturnSum->mlTotal, sc1), mpReturnSum->miCount);
	StringTemp = StringConv;
	if ((mpReturnSum->miSubDir)) {
		sprintf (StringConv, "%7i %7.7s %7i ", mpReturnSum->miSubDir,
						 conv (mpReturnSum->mlSTotal, sc2), mpReturnSum->miSFiles);
	} else
		sprintf (StringConv, "%24.24s", "");
	StringTemp += StringConv;

	if ((mpReturnSum->miSSubDir)) {
		sprintf (StringConv, "%7i %7.7s %7i ", mpReturnSum->miSSubDir,
						 conv (mpReturnSum->mlSSTotal, sc2), mpReturnSum->miSSFiles);
	} else
		sprintf (StringConv, "%24.24s", "");
	StringTemp += StringConv;

	sprintf (StringConv, "%7i %7.7s", totfiles,
					 conv (mpReturnSum->mlNetTotal, sc3));
	StringTemp += StringConv;

	*mpOutputString[miCountDirs] = StringTemp;
}

char *SubDirectory::conv (ulnglng in, char *tstr)
{
	//double tera=1099511627776;
	double tera = 1.0995116e12;

	if (in < 1024) {
		sprintf (tstr, "%5.1lli ", in);
	}
	if ((in >= 1024) && (in < 1048576)) {
		double x = double (in) / 1024;

		sprintf (tstr, "%5.1fK", x);
	}
	if ((in >= 1048576) && (in < 1073741824)) {
		double x = double (in) / 1048576;

		sprintf (tstr, "%5.1fM", x);
	}
	if ((in >= 1073741824) && (double (in) < tera)) {
		double x = double (in) / 1073741824;

		sprintf (tstr, "%5.1fG", x);
	}
	if (double (in) >= tera) {
		double x = double (in) / tera;

		sprintf (tstr, "%5.1fT", x);
	}
	return tstr;
}

char *SubDirectory::conv (int in, char *tstrr)
{
	ulnglng y;

	y = (ulnglng) in;
	conv (y, tstrr);
	return tstrr;
}

/*char* SubDirectory::conv(ulnglng in,char* tstrr)
{
	ulnglng y;
	y=(ulnglng)in;
	conv(y,tstrr);
	return tstrr;
}*/

/*void SubDirectory::check(int c){
	ulnglng sumary[22];
	float percentary[22];
	int i;
	ulnglng test=0;
	for( i=0; i<c; i++){
		ulnglng sumtst=0;
		ulnglng sum = ++( *mpRetVal[i] );
		sumary[i]=sum;
		sumtst+=mpRetVal[i]->mlSSTotal;
		sumtst+=mpRetVal[i]->mlSTotal;
		sumtst+=mpRetVal[i]->mlTotal;
		if( sumtst != sum ){
			int whop=0;
			whop++;
		}
		test+=sumtst;
		percentary[i]=( 100 * (double)sumtst / (double)mpReturnSum->mlNetTotal ); //(double)
	}
	test+=mpReturnSum->mlBytes;
	if( test != mpReturnSum->mlNetTotal ){
		int whop=0;
		whop++;
	}
	return;
}*/
